package com.javamonkey.test;

/*
 * Copyright (c) 1999 Peter Seibel
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

import com.javamonkey.utils.AssertionError;
import com.javamonkey.utils.Debug;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * Base class for tests. Provides support for everything in the Test
 * interface but the <code>run()</code> method itself. Also provides
 * convenience methods for subclasses to use to report results.
 */
public abstract class BaseTest implements Test, TestListener {

  private final static boolean debug = true;
  
  private final static String NAME_NOT_SET = new String();
  private final static int NUM_NOT_SET = -1;

  /**
   * The set of TestListeners that we fire events at.
   */
  private Set listeners = new HashSet();

  /**
   * The name of the Test. Defaults to the Test's class name.
   */
  private String name = NAME_NOT_SET;

  /**
   * The number of test cases in the Test.
   */
  private int numberOfTestCases = NUM_NOT_SET;

  /**
   * Make a new BaseTest with a default name. (Note: the subclass must
   * call setNumberOfTestCases() before the first test event is fired.)
   */
  public BaseTest() {}

  /**
   * Make a new Test with the given name and number of test cases.
   */
  public BaseTest(String n, int cases) {
    name = n;
    numberOfTestCases = cases;
  }

  /**
   * Make a new Test with the class name as the name and the given
   * number of test cases.
   */
   public BaseTest(int cases) {
     name = getClass().getName();
     numberOfTestCases = cases;
   }

  /**
   * The guts of the test -- this must be implemented by concrete
   * subclasses.
   */
  public abstract void run();

  /**
   * Called after run() completes. Subclasses can override this to
   * clean up any non-memory resources they used.
   */
  public void cleanup() {}

  /**
   * Get the name of this Test.
   */
  public String getName() {
    if (debug) Debug.assert(name != NAME_NOT_SET);
    return name;
  }

  /**
   * Set the name of this Test. Used by subclasses that don't know
   * their name when they're constructed.
   */
  protected void setName(String n) {
    name = n;
  }

  /**
   * Get the number of test cases in this Test.
   */
  public int getNumberOfTestCases() {
    if (debug) Debug.assert(numberOfTestCases != NUM_NOT_SET);
    return numberOfTestCases;
  }

  /**
   * Set the number of test cases in this Test. Used by subclasses
   * that don't know when they're constructed how many test cases they
   * will generate.
   */
  protected void setNumberOfTestCases(int num) {
    numberOfTestCases = num;
  }

  /**
   * Add a TestListener.
   */
  public void addTestListener(TestListener l) {
    listeners.add(l);
  }

  /**
   * Remove a TestListener.
   */
  public void removeTestListener(TestListener l) {
    listeners.remove(l);
  }

  /**
   * Fire a TestEvent at all the listeners indicating a test case success.
   */
  protected void reportSuccess(String name, String msg) {
    fireTestEvent(getName() + "!" + name, SUCCESS, msg, null);
  }

  /**
   * Fire a TestEvent at all the listeners indicating a test case failure.
   */
  protected void reportFailure(String name, String msg) {
    fireTestEvent(getName() + "!" + name, FAILURE, msg, null);
  }

  protected void reportFailure(String name, String msg, Throwable t) {
    fireTestEvent(getName() + "!" + name, FAILURE, msg, t);
  }

  /**
   * Fire a TestEvent at all the listeners indicating a skipped test case.
   */
  protected void reportSkip(String name) {
    fireTestEvent(getName() + "!" + name, SKIPPED, (String)null, (Throwable)null);
  }

  protected void reportSkip(String name, String msg) {
    fireTestEvent(getName() + "!" + name, SKIPPED, msg, (Throwable)null);
  }

  protected void reportSkip(String name, Throwable t) {
    fireTestEvent(getName() + "!" + name, SKIPPED, (String)null, t);
  }

  /**
   * Fire a TestEvent at all the listeners indicating an aborted test case.
   */
  protected void reportAbort(String name) {
    fireTestEvent(getName() + "!" + name, ABORTED, (String)null, (Throwable)null);
  }

  /**
   * Fire a TestEvent at all the listeners indicating an aborted test case.
   */
  protected void reportAbort(String name, Throwable t) {
    fireTestEvent(getName() + "!" + name, ABORTED, (String)null, t);
  }

  /**
   * Fire a TestEvent to all the listeners.
   */
  private void fireTestEvent(String name, int code, String msg, Throwable t) {
    TestEvent evt = new TestEvent(this, name, code, msg, t);
    fireTestEvent(evt);
  }

  private void fireTestEvent(TestEvent evt) {
    Iterator i = listeners.iterator();

    while (i.hasNext()) {
      TestListener l = (TestListener)i.next();
      switch (evt.getResult()) {
      case Test.SUCCESS: l.testSucceeded(evt); break;
      case Test.FAILURE: l.testFailed(evt); break;
      case Test.SKIPPED: l.testSkipped(evt); break;
      case Test.ABORTED: l.testAborted(evt); break;
      default:
	throw new AssertionError(
	  "Illegal test result code: " + evt.getResult());
      }
    }
  }

  // =================================================================
  // TestListener implementation -- to support tests that create
  // sub-tests we are a listener for TestEvents as well as a source:
  // parent tests can be added as listeners to their children and the
  // results will propogate up the chain. (Note: if the trees of tests
  // are deep then this is not the most efficient mechanism, better
  // perhaps, to copy the parents listeners to the children. But that
  // assumes that the parents have their listeners added before they
  // make their children which might not be true. So, this is good
  // enough for now.)
    
  public void testSucceeded(TestEvent evt) {
    fireTestEvent(evt);
  }
  public void testFailed(TestEvent evt) {
    fireTestEvent(evt);
  }
  public void testSkipped(TestEvent evt) {
    fireTestEvent(evt);
  }
  public void testAborted(TestEvent evt) {
    fireTestEvent(evt);
  }

  // End TestListener implementation
  // =================================================================

}







