package com.javamonkey.test;

/*
 * Copyright (c) 1999 Peter Seibel
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

import java.util.EventObject;

/**
 * Event object representing the result of a single test case.
 * TestEvents are listened for by the TestListener interface. Any Test
 * class that extends from {@link BaseTest BaseTest} can fire these
 * events via the various <code>reportX()</code> methods.
 *
 * @see com.javamonkey.test.TestListener
 * @see com.javamonkey.test.BaseTest#report

 */
public class TestEvent extends EventObject {

  public final static long NO_TIMESTAMP = -1L;
  private static boolean timestampsOn = true;

  private String    name;
  private int       result;
  private String    message;
  private Throwable throwable;
  private long      timestamp = NO_TIMESTAMP;
  
  /**
   * Create a TestEvent representing a test case.
   *
   * @param source The Test that the test case is a part of.
   * @param name The simple name of the test case.
   * @param result The result code for the test case.
   * @param msg A human readable message.
   * @param t A Throwable associated with the test result.
   */ 
  public TestEvent(Test s, String n, int r, String msg, Throwable t) {
    super(s);
    name      = n;
    result    = r;
    message   = msg;
    throwable = t;
    if (timestampsOn) timestamp = System.currentTimeMillis();
  }

  /**
   * Turn on timestamping of TestEvents. Causes a call to
   * <code>System.currentTimeMillis()</code> in each constructor.
   */
  public static void turnOnTimestamping() {
    timestampsOn = true;
  }

  /**
   * Turn off timestamping of TestEvents. Causes a call to
   * <code>System.currentTimeMillis()</code> in each constructor.
   */
  public static void turnOffTimestamping() {
    timestampsOn = false;
  }

  /**
   * Get the name of the test case.
   */
  public String getName() {
    return name;
  }

  /**
   * Return the result: one of SUCCESS, FAILURE, SKIPPED, or ABORTED
   * defined in the Test interface.
   */
  public int getResult() {
    return result;
  }

  public String getMessage() {
    return message;
  }

  /**
   * Return the Throwable (if any) associated with this
   * TestEvent. Otherwise return null.
   */
  public Throwable getThrowable() {
    return throwable;
  }

  /**
   * Get the timestamp.
   */
  public long getTimestamp() {
    return timestamp;
  }
}




