package com.javamonkey.utils.reflect;

/*
 * Copyright (c) 1999 Peter Seibel
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

import com.javamonkey.utils.AssertionError;
import com.javamonkey.utils.Debug;
import java.lang.reflect.*;
import java.util.*;

/**
 * Utility class for conveniently using reflection to access fields and
 * methods including private members.
 *
 * @version $Id: //depot/macaca/src/com/javamonkey/utils/reflect/Reflector.java#1 $
 */
public class Reflector {

  private final static boolean debug = true;

  
  public boolean booleanField(String name, Object o) 
  {
    try {
      return findField(name, o).getBoolean(o);
    } catch (IllegalAccessException iae) {
      throw new AssertionError(iae);
    }
  }

  public byte byteField(String name, Object o) 
  {
    try {
      return findField(name, o).getByte(o);
    } catch (IllegalAccessException iae) {
      throw new AssertionError(iae);
    }
  }

  public char charField(String name, Object o) 
  {
    try {
      return findField(name, o).getChar(o);
    } catch (IllegalAccessException iae) {
      throw new AssertionError(iae);
    }
  }

  public double doubleField(String name, Object o) 
  {
    try {
      return findField(name, o).getDouble(o);
    } catch (IllegalAccessException iae) {
      throw new AssertionError(iae);
    }
  }

  public float floatField(String name, Object o) 
  {
    try {
      return findField(name, o).getFloat(o);
    } catch (IllegalAccessException iae) {
      throw new AssertionError(iae);
    }
  }

  public int intField(String name, Object o) 
  {
    try {
      return findField(name, o).getInt(o);
    } catch (IllegalAccessException iae) {
      throw new AssertionError(iae);
    }
  }

  public long longField(String name, Object o) 
  {
    try {
      return findField(name, o).getLong(o);
    } catch (IllegalAccessException iae) {
      throw new AssertionError(iae);
    }
  }

  public short shortField(String name, Object o) 
  {
    try {
      return findField(name, o).getShort(o);
    } catch (IllegalAccessException iae) {
      throw new AssertionError(iae);
    }
  }


  private Field findField(String name, Object o) {
    if (debug) {
      Debug.assert(name != null);
      Debug.assert(name.length() > 0);
      Debug.assert(o != null);
    }
    
    Class c = o.getClass();
    while (c != null) {
      Field[] fields = c.getDeclaredFields();
      for (int i = 0; i < fields.length; i++) {
	if (fields[i].getName().equals(name)) {
	  fields[i].setAccessible(true);
	  return fields[i];
	}
      }
      c = c.getSuperclass();
    }
    
    throw new ReflectorException("Field '" + name + "' not found.");
  }

  /**
   * Inovke the given named method on the given object with the given
   * args. Can be used to invoke private methods.
   */
  public Object invoke(String name, Object o, Object[] args) 
    throws InvocationException, Exception, SecurityException
  {
    if (debug) { Debug.assert(o != null); }

    Method m = findMethod(name, o.getClass(), args);
    try {
      return m.invoke(o, args);

    } catch (InvocationTargetException ite) {
      Throwable t = ite.getTargetException();
      if (t instanceof Exception) {
	throw (Exception)t;
      } else {
	try {
	  throw (Error)t;
	} catch (ClassCastException cce) {
	  throw new AssertionError(
	    "Throwable subclass " + t.getClass().getName() + " neither an " +
	    "Exception nor an Error");
	}

      }
    } catch (IllegalArgumentException iae) {
      // We can't get here unless findMethod() gave us a bad method.
      throw new AssertionError(iae);

    } catch (IllegalAccessException iae) {
      // Likewise, this can't happen unless findMethod() has given us an
      // inaccessible object.
      throw new AssertionError(iae);
    }
  }
  
  /**
   * Find a method with the given name on the given class that can
   * accept the given arguments.
   *
   * @throws AmbiguousInvocationException if more than one compatible
   * method is found.
   */
  private Method findMethod(String name, Class c, Object[] args) 
    throws InvocationException
  {
    Class[] argClasses = getArgumentClasses(args);
    Collection methods = getAllMethodsWithName(c, name);
    if (methods.isEmpty()) {
      throw new NoMethodFoundException(
	"No methods named " + name + " on " + c);
    }
    Iterator i = methods.iterator();
    Method result = null;
    while (i.hasNext()) {
      Method m = (Method)i.next();
      if (argumentsAreCompatible(m, argClasses)) {
	if (result != null) {
	  throw new AmbiguousInvocationException(
	    m + " and " + result + " both compatible.");
	} else {
	  result = m;
	}
      }
    }
    if (result == null) {
      throw new NoMethodFoundException(
	"No compatible method " + name + " found on class " + c);
    } else {
      result.setAccessible(true);
      return result;
    }
  }

  private boolean argumentsAreCompatible(Method m, Class[] argClasses) {
    Class[] paramTypes = m.getParameterTypes();

    if (paramTypes.length != argClasses.length) return false;

    for (int i = 0; i < paramTypes.length; i++) {
      if (!argumentTypeIsCompatible(paramTypes[i], argClasses[i])) {
	return false;
      }
    }
    return true; // made it.
  }

  private boolean argumentTypeIsCompatible(Class paramType, Class argType)
  {
    if (paramType.isPrimitive()) {
      paramType = primitiveTypeToWrapperType(paramType);
    }
    
    return paramType.isAssignableFrom(argType);
  }

  private Class primitiveTypeToWrapperType(Class type) {
    if (type == Boolean.TYPE) {
      return Boolean.class;
    } else if (type == Byte.TYPE) {
      return Byte.class;
    } else if (type == Character.TYPE) {
      return Character.class;
    } else if (type == Double.TYPE) {
      return Double.class;
    } else if (type == Float.TYPE) {
      return Float.class;
    } else if (type == Integer.TYPE) {
      return Integer.class;
    } else if (type == Long.TYPE) {
      return Long.class;
    } else if (type == Short.TYPE) {
      return Short.class;
    } else {
      throw new AssertionError(type + " is not primitive.");
    }
  }

  private Class[] getArgumentClasses(Object[] objs) {
    Class[] classes = new Class[objs.length];
    for (int i = 0; i < objs.length; i++) {
      classes[i] = objs[i].getClass();
    }
    return classes;
  }

  private Collection getAllMethodsWithName(Class c, String name) {
    if (debug) Debug.assert(c != null);

    Collection result = new HashSet();

    while (c != null) {
      Method[] methods = c.getDeclaredMethods();
      for (int i = 0; i < methods.length; i++) {
	Method m = methods[i];
	if (m.getName().equals(name)) {
	  result.add(m);
	}
      }
      c = c.getSuperclass();
    }
    return result;
  }


}







